using System;
using Server.Items;

namespace Server.Engines.Harvest
{
	public class DefMining : HarvestSystem
	{
		public override SkillName MainSkill
		{
			get	{ return SkillName.Mining; }
		}

		private static HarvestSystem m_HarvestSystem;

		public static HarvestSystem HarvestSystem
		{
			get
			{
				if ( m_HarvestSystem == null )
					m_HarvestSystem = new DefMining();

				return m_HarvestSystem;
			}
		}

		private DefMining() : base( 2, 2, 1.6 )
		{
		}

		public override bool CanHarvest( Mobile from, Item item, HarvestItem harvestItem )
		{
			if ( from.Mounted )
			{
				from.SendLocalizedMessage( 503038 ); // You can't mine while riding
				return false;
			}
			
			if ( from.FindItemOnLayer( Layer.OneHanded ) != item )
			{
				from.SendMessage( "The pickaxe must be equipped for any serious miner" );
				return false;
			}

			if ( harvestItem == null )
			{
				from.SendLocalizedMessage( 503036 ); // You can't mine there
				return false;
			}

			return true;
		}

		public override void PlayHarvestEffect( Mobile from )
		{
			if ( from.Body.Type == BodyType.Human && !from.Mounted )
				from.Animate( 11, 5, 1, true, false, 0 );

			new InternalTimer( from ).Start();
		}

		// Delay to synchronize the sound with the hit on the tree
		private class InternalTimer : Timer
		{
			private Mobile m_From;

			public InternalTimer( Mobile from ) : base( TimeSpan.FromSeconds( 0.9 ) )
			{
				m_From = from;
			}

			protected override void OnTick()
			{
				m_From.PlaySound( Utility.Random( 0x125, 2 ) );
			}
		}

		public override void PlayEndingEffect( Mobile from, bool failed, bool toBackpack, bool noMoreRes, bool hasMovedAway )
		{
			if ( hasMovedAway )
			{
				from.SendLocalizedMessage( 501867 ); // You have moved too far away to continue mining
				return;
			}
			
			// Someone has gotten the ressource before you
			if ( noMoreRes )
			{
				from.SendLocalizedMessage( 503042 ); // Someone has gotten the metal before you
				return;
			}
			
			if ( failed )
			{
				from.SendLocalizedMessage( 503043 ); // You loosen some rocks but fail to find any useable ore
			}
			else
			{
				if ( toBackpack )
				{
					from.SendLocalizedMessage( 503044 ); // You dig some ore and put it in your backpack
				}
				else
				{
					from.SendLocalizedMessage( 503045 ); // You dig some ore but you have no place to put it

				}
			}
		}

		public override void InitHarvestList()
		{
			/*
			 * ItemID		: Item we can harvest from
			 * MinTotalRes	: Minimum amount of resource the item can have
			 * MaxTotalRes	: Maximum amount of resource the item can have
			 * MinSkill		: Minimum skill value you need to be able to harvest
			 * MaxSkill		: Maximum skill value to harvest
			 * TypeRes		: Type of the ressource you will get to harvest this item
			 * MinAmount	: Minimum amount of ressource you can get from this item in one hit
			 * MaxAmount	: Maximum amount of ressource you can get from this item in one hit
			 * RegenDelay	: Delay for the item to regenarate when the item has no more ressource
			 * Message		: The message the mobile will receive when no more ressource available
			 */
			
			int[] CaveTileIDs = new int[]
				{ 0x53B, 0x53C, 0x53D, 0x53E, 0x53F, 0x540, 0x541, 0x542,
				  0x543, 0x544, 0x545, 0x546, 0x547, 0x548, 0x549, 0x54a,
				  0x54b, 0x54c, 0x54d, 0x54e, 0x54f };
			
			AddHarvest( CaveTileIDs, 9, 25, 0, 80, typeof( IronOre ), 2, 5, TimeSpan.FromMinutes( 15 ), 503040 );

			
			int [] MountainTileIDs = new int[]
				{ 570, 571, 297, 296, 277, 278, 222, 223, 543,
				  544, 545, 546, 547, 548, 549, 550, 551, 552,
				  553, 554, 555, 556, 557, 558, 559, 560, 236,
				  237, 238, 239 };
			
			AddHarvest( MountainTileIDs, 9, 25, 0, 80, typeof( IronOre ), 2, 5, TimeSpan.FromMinutes( 15 ), 503040 );


			/*
			 * TypeRes		: Type of the resource to define
			 * MinTotalRes	: Minimum amount of ressource the item can have
			 * MaxTotalRes	: Maximum amount of ressource the item can have
			 * MinSkill		: Minimum skill value you need to be able to harvest
			 * MaxSkill		: Maximum skill value to harvest
			 */

			// Definition of the minable ore
			AddResourceInfo( typeof ( DullCopperOre ), 8, 20, 65.0, 100.0 );
			AddResourceInfo( typeof ( ShadowIronOre ), 7, 17, 70.0, 100.0 );
			AddResourceInfo( typeof ( CopperOre ), 6, 14, 75.0, 100.0 );
			AddResourceInfo( typeof ( BronzeOre ), 5, 12, 80.0, 100.0 );
			AddResourceInfo( typeof ( GoldOre ), 4, 10, 85.0, 100.0 );
			AddResourceInfo( typeof ( AgapiteOre ), 4, 8, 90.0, 100.0 );
			AddResourceInfo( typeof ( VeriteOre ), 3, 6, 95.0, 100.0 );
			AddResourceInfo( typeof ( ValoriteOre ), 2, 4, 99.0, 100.0 );
		}
	}
}